@@ -1,6 +1,7 @@
 """Main module."""
 
 import asyncio
+from copy import deepcopy
 import logging
 import re
 import sys
@@ -37,6 +38,7 @@ from aiosonic.exceptions import (
     MaxRedirects,
     MissingWriterException,
     ReadTimeout,
+    ConnectionDisconnected,
     RequestTimeout,
     TimeoutException,
 )
@@ -63,7 +65,7 @@ _CHUNK_SIZE = 1024 * 4  # 4kilobytes
 _NEW_LINE = "\r\n"
 dlogger = get_debug_logger()
 
-REPLACEABLE_HEADERS = {'host', 'user-agent'}
+REPLACEABLE_HEADERS = {"host", "user-agent"}
 
 
 # Functions with cache
@@ -105,7 +107,7 @@ def _headers_iterator(headers: HeadersType):
 def _add_headers(headers: HeadersType, headers_to_add: HeadersType):
     """Safe add multiple headers."""
     for key, data in _headers_iterator(headers_to_add):
-        replace = key.lower() in  REPLACEABLE_HEADERS
+        replace = key.lower() in REPLACEABLE_HEADERS
         _add_header(headers, key, data, replace)
 
 
@@ -466,6 +468,7 @@ async def _do_request(
     ssl: Optional[SSLContext],
     timeouts: Optional[Timeouts],
     http2: bool = False,
+    times: int = 0,
 ) -> HttpResponse:
     """Something."""
     timeouts = timeouts or connector.timeouts
@@ -497,7 +500,9 @@ async def _do_request(
                 raise HttpParsingError(f"response line parsing error: {line}")
             response._set_response_initial(line)
         except asyncio.IncompleteReadError as exc:
-            raise HttpParsingError(f"response line parsing error: {exc.partial}")
+            connection.keep = False
+            raise ConnectionDisconnected(times)
+            # raise HttpParsingError(f"response line parsing error: {exc.partial}")
         except TimeoutException:
             raise ReadTimeout()
 
@@ -579,7 +584,7 @@ class HTTPClient:
             TypeError('missing argument, either "json" or "data"')
         if json is not None:
             data = json_serializer(json)
-            headers = headers or HttpHeaders()
+            headers = deepcopy(headers) if headers else HttpHeaders()
             _add_header(headers, "Content-Type", "application/json")
         return await self.request(
             url,
@@ -781,7 +786,7 @@ class HTTPClient:
         urlparsed = _get_url_parsed(url)
 
         boundary = None
-        headers = headers or []
+        headers = deepcopy(headers) if headers else []
         body: ParsedBodyType = b""
 
         if self.handle_cookies:
@@ -798,7 +803,8 @@ class HTTPClient:
         max_redirects = 30
         # if class or request method has false, it will be false
         verify_ssl = verify and self.verify_ssl
-        while True:
+        reconnect_times = 3
+        while reconnect_times > 0:
             headers_data = partial(
                 _prepare_request_headers,
                 url=urlparsed,
@@ -846,10 +852,13 @@ class HTTPClient:
                         )
                 else:
                     return response
+            except ConnectionDisconnected:
+                reconnect_times -= 1
             except ConnectTimeout:
                 raise
             except TimeoutException:
                 raise RequestTimeout()
+        raise ConnectionDisconnected("retried 3 times unsuccessfully")
 
     async def wait_requests(self, timeout: int = 30):
         """Wait until all pending requests are done.
